package de.tudarmstadt.ke.sw.matching.wikimatch.matcher;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.semanticweb.owl.align.Alignment;
import org.semanticweb.owl.align.AlignmentException;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.vocabulary.RDFS;

import de.tudarmstadt.ke.sw.matching.wikimatch.Helper;
import de.tudarmstadt.ke.sw.matching.wikimatch.Matcher;
/**
 * @author Sven Hertling
 * 
 */
public class OaeiWikiMatch implements Matcher {

	@Override
	public Alignment align(OntModel onto1, OntModel onto2, Alignment alignment) throws AlignmentException {		
		
		String defLanguage1 =  getDefaultLanguage(onto1);
		String defLanguage2 =  getDefaultLanguage(onto2);
		
		ResourceMatcher.setLanguage1(defLanguage1);
		ResourceMatcher.setLanguage2(defLanguage2);
		
		double threshold;
		
		if(defLanguage1.equals(defLanguage2)){
			threshold = 0.8d;
		}
		else{
			threshold = 0.06d;
		}		
		
		iterateOver(onto1.listClasses().toList(), onto2.listClasses().toList(), alignment, threshold);
		iterateOver(onto1.listDatatypeProperties().toList(), onto2.listDatatypeProperties().toList(), alignment, threshold);
		iterateOver(onto1.listObjectProperties().toList(), onto2.listObjectProperties().toList(), alignment, threshold);
		
		return alignment;
	}
	
	
	
	private void iterateOver(List<? extends OntResource> listOne, List<? extends OntResource> listTwo, Alignment m, double thresould) throws AlignmentException{
		Iterator<? extends OntResource> iteratorOne = listOne.iterator();        
		while(iteratorOne.hasNext()) {   
        	if(ResourceMatcher.set1(iteratorOne.next())){
        		Iterator<? extends OntResource> iteratorTwo = listTwo.iterator();        
		    	while(iteratorTwo.hasNext()) {  		    		
		    		if(ResourceMatcher.set2(iteratorTwo.next())){
		    			double confidence = ResourceMatcher.match();
		    			if(confidence >= thresould)
		    				Helper.addAlignCell(m, ResourceMatcher.getUri1(), ResourceMatcher.getUri2(), confidence);
		    		}
		        }		    	
        	}
		}
	}
	
	public String getDefaultLanguage(OntModel m) {
		
        Map<String,Integer> map = new HashMap<String,Integer>();
        ResIterator it = m.listResourcesWithProperty(RDFS.label);
        
        while(it.hasNext()) {
            StmtIterator labels = it.next().listProperties(RDFS.label);
            while(labels.hasNext()) {            	
                String language = labels.next().getLiteral().getLanguage();
                if(language.length() > 0){
                	Integer count = map.get(language);
        		    map.put(language, count != null ? count+1 : 1);
                }
            }
        }
        
        // maximum
        String languageMax = new String("en");
        int countMax = 0;
        for(Entry<String,Integer> entry : map.entrySet()) {
            if(entry.getValue() > countMax) {
            	languageMax = entry.getKey();
            	countMax = entry.getValue();
            }
        }
        
        if(languageMax.equals("cn")){
        	return "zh";
        }
        if(languageMax.equals("cz")){
        	return "cs";
        }        
        return languageMax;
    }

	@Override
	public void setMaxConfidence(double maxConfidence) {}

}
