package de.tudarmstadt.ke.sw.matching.wikimatch.matcher;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.RDFNode;
/**
 * @author Sven Hertling
 * 
 */
public class Resource implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -9038311016337428681L;
	
	private SubResource name;
	private Set<SubResource> labels;
	private Set<SubResource> comments;
	private String uri;
	private String defaultLanguage;
	
	public static Resource create(OntResource res, String defaultLang){
		
		Resource r = TimeSaver.getResource(res.getURI());
		if(r != null){
			return r;
		}
		
		Resource resource = null;
		
		String localName = res.getLocalName();
		if(localName == null)
			localName = new String();
		
		Set<RDFNode> listLabel = res.listLabels(null).toSet();			
		Set<RDFNode> listComments = res.listComments(null).toSet();
		
		if(localName.length() != 0 || listLabel.size() != 0 || listComments.size() != 0){
			resource = new Resource(res.getURI(), localName, listLabel, listComments, defaultLang);
			TimeSaver.put(res.getURI(), resource);
		}
		return resource;
	}
	
	
	private Resource(String _uri, String _name, Set<RDFNode> _labels, Set<RDFNode> _comments, String _defaultLang){
		this.uri = _uri;
		this.defaultLanguage = _defaultLang;
		
		this.name = SubResource.create(_name, defaultLanguage);
		
		this.labels = new HashSet<SubResource>();
		for(RDFNode node : _labels){			
			this.labels.add(SubResource.create(node.asLiteral().getString(), getLanguage(node, defaultLanguage)));
		}
		
		this.comments = new HashSet<SubResource>();
		for(RDFNode node : _comments){
			this.comments.add(SubResource.create(node.asLiteral().getString(), getLanguage(node, defaultLanguage)));
		}
	}
	
	private String getLanguage(RDFNode node, String defaultLang){
		if(node.asLiteral().getLanguage().length() != 0){
			String lang = node.asLiteral().getLanguage();
			if(lang.equals("cn")){
	        	return "zh";
	        }
	        if(lang.equals("cz")){
	        	return "cs";
	        }
	        return lang;
		}
		else{
			return defaultLang;
		}
	}
	
	
	private Set<SubResource> getAll(){
		Set<SubResource> all = new HashSet<SubResource>();
		all.add(name);
		all.addAll(labels);
		all.addAll(comments);
		return all;
	}
	
	public static double match(Resource a, Resource b){
		
		Set<SubResource> setA = a.getAll();
		Set<SubResource> setB = b.getAll();
		
		//find maximum comments - comments, comments - labels , comments - localname, etc.
		double max = 0.0d;
		for(SubResource oneA : setA){
			for(SubResource oneB : setB){
				double tmp = SubResource.match(oneA, oneB);
				if(tmp > max){
					max = tmp;
				}
			}
		}
		return max;
	}
	
	
	public String toString() {
		return this.name.toString();
	}
	
	
	public String getURI(){
		return uri;
	}
}
