package de.tudarmstadt.ke.sw.matching.wikimatch.matcher;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * @author Sven Hertling
 * 
 */
public class SubResource implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 4000960547787667429L;
	
	private String text;	
	private String language;
	private URL wiki;
	
	private Map<String, Titles> languageMap;

	public SubResource(String _text, String _language) {
				
		this.text = _text;
		this.language = _language;
		this.languageMap = new HashMap<String, Titles>();
		//askWiki

		try {
			wiki = new URL("http://" + language + ".wikipedia.org/w/api.php");
		} catch (MalformedURLException e1) {e1.printStackTrace();}
		
		//ask for titles in language "this.language"
		ArrayList<String> inlanguage = askForTitles(this.text, 2);
		
		if(inlanguage.size() == 0){
			return;
		}
		
		//put these tiles in languageMap with language "this.language"
		Titles levelfour = new Titles(inlanguage);
		this.languageMap.put(this.language, levelfour);
		
		askForTranslated(inlanguage);
	}
		
	
	public static double match(SubResource a, SubResource b){			
		double union = 0.0d;
		double intersection = 0.0d;		
		for(Entry<String, Titles> entry : a.languageMap.entrySet()){
			if(b.languageMap.containsKey(entry.getKey())){
				//same language				
				/*
				double tmp = LevelFour.match(entry.getValue(), b.languageMap.get(entry.getKey()));
				if(tmp > max){
					max = tmp;
					languagemax = entry.getKey();
				}
				*/
				intersection += Titles.getintersection(entry.getValue(), b.languageMap.get(entry.getKey()));
				union += Titles.getunion(entry.getValue(), b.languageMap.get(entry.getKey()));
			}
		}
		return intersection / union;
	}	
	

	private ArrayList<String> askForTitles(String _text, int accumulator) {
		if(accumulator <= 0){
			return new ArrayList<String>();
		}
		ArrayList<String> list = new ArrayList<String>();
		WikipediaRequest request = new WikipediaRequest(wiki);
		
		request.putParameter("action", "query");
		request.putParameter("list", "search");
		request.putParameter("srinfo", "suggestion");//totalhits, suggestion
		request.putParameter("srlimit", "50");//49
		request.putParameter("srprop", "");//snippet
		request.putParameter("srsearch", _text);
		request.putParameter("srwhat", "text");	
		request.putParameter("srredirects", "true");	
		request.putParameter("format", "json");

		String json = request.runRequest();
		
		//System.out.println("titles:");
		//System.out.println(request.getURL());
		//System.out.println(json);		
		
		try {
			JSONObject answer = new JSONObject(json);
			printPossibleWarningsErros(answer);
			
			JSONObject query = answer.getJSONObject("query");
			JSONArray search = query.getJSONArray("search");
			for(int i = 0; i < search.length(); i++){
				JSONObject oneSearch = search.getJSONObject(i);
				list.add(oneSearch.getString("title").trim());				
	        }
			
			//check for suggestion - recursive
			if(query.has("searchinfo")){
				JSONObject searchinfo = query.getJSONObject("searchinfo");
				if(searchinfo.has("suggestion")){
					list.addAll(askForTitles(searchinfo.getString("suggestion"), accumulator - 1));
				}
			}
			
		} catch (JSONException e) {
			e.printStackTrace();
		}		
		return list;
	}
	
	
	private void askForTranslated(ArrayList<String> list){		
		Titles inLanguage = this.languageMap.get(this.language);		
		
		Iterator<String> it = list.iterator();		
		while(it.hasNext()){
			StringBuilder sb = new StringBuilder();
			for(int i = 0; i < 50; i++){
				if(it.hasNext()){
					sb.append(it.next().trim());
					sb.append("|");
				}
				else{
					break;					
				}
			}
			String tmpTitles = sb.toString();
			final String wikiTitels = tmpTitles.substring(0, tmpTitles.lastIndexOf("|"));
			askForTranslated(wikiTitels, inLanguage);
		}
	}
	
	private void askForTranslated(String wikiTitels, Titles inLanguage){
		String llcontinue = new String();
		do{		
			WikipediaRequest request = new WikipediaRequest(wiki);
			request.putParameter("action", "query");
			request.putParameter("prop", "langlinks");
			request.putParameter("lllimit", "500");
			request.putParameter("titles", wikiTitels);
			request.putParameter("format", "json");
			if(llcontinue.length() != 0)
				request.putParameter("llcontinue", llcontinue);
			
			String json = request.runRequest();
			
			//System.out.println("translated:");
			//System.out.println(request.getURL());
			//System.out.println(json);

			
			try {		
				
				JSONObject answer = new JSONObject(json);
				printPossibleWarningsErros(answer);
				JSONObject query = answer.getJSONObject("query");	
				JSONObject pages = query.getJSONObject("pages");
				
				String[] array = JSONObject.getNames(pages);
				
				for(int i = 0; i < array.length ; i++){
					JSONObject tmp = pages.getJSONObject(array[i]);
					
					if(Integer.parseInt(array[i]) < 0){
						System.out.println("ERROR:  with" + tmp.toString());
					}
					
					
					if(tmp.has("langlinks")){
						
						int index = inLanguage.getIndex(tmp.getString("title"));
						
						if(index < 0){
							System.out.println("ERROR: keine title: " + request.getURL());
							continue;
						}
						
						JSONArray langlinks = tmp.getJSONArray("langlinks");
						for(int ll = 0; ll < langlinks.length(); ll++){
							JSONObject oneLangLink = langlinks.getJSONObject(ll);
							
							Titles languageToAdd = this.languageMap.get(oneLangLink.getString("lang"));
							if(languageToAdd == null){
								languageToAdd = new Titles(Arrays.asList(oneLangLink.getString("*")));					
								this.languageMap.put(oneLangLink.getString("lang"), languageToAdd);
							}
							else{
								languageToAdd.add(oneLangLink.getString("*"), index);
							}
				        }
					}
				}
				
				if(answer.has("query-continue")){
					llcontinue = answer.getJSONObject("query-continue").getJSONObject("langlinks").getString("llcontinue");
				}
				else
					llcontinue = new String();
				
			} catch (JSONException e) {
				e.printStackTrace();
			}
			
		}while(llcontinue.length() != 0);
	}




	private void printPossibleWarningsErros(JSONObject object) throws JSONException{
		if(object.has("error")){
			System.out.println("ERROR: " + object.get("error").toString());
		}
		if(object.has("warnings")){
			System.out.println("WARNING: " + object.get("warnings").toString());
		}
	}
	
	public String toString(){
		return text.concat(" @ ").concat(language);
	}
	

	public String getText() {
		return text;
	}
	
	//Save already asked queries	
	private static HashMap<String, SubResource> map = new HashMap<String, SubResource>();
	public static SubResource create(String _text, String _language){
		if(map.containsKey(_text)){
			//System.out.println(_text + " -> found");
			return map.get(_text);
		}
		else{
			String search = MyStringUtil.makeWikiString(_language, _text);
			//System.out.println(_text + " -> search" + " -> " + search);			
			SubResource sr = new SubResource(search, _language);
			map.put(_text, sr);			
			return sr;
		}
	}
}
